root_path = File.dirname(__FILE__)
require "socket"
require "#{root_path}/../config"

class IPSocket
	def info; peeraddr[1,2].reverse.join(':'); end
end

# TODO: implement sort of a better config scheme
module Logging
	require "logger"
	@@log = Logger.new(Log_to)

	Log_level ||= Logger::INFO
	@@log.level = Log_level

	def log(level, message)
		@@log.send(level, message)
	end
end

# TODO: parse and generate automatically based on parameters and
#		their type
class Message

	# format_string example :
	#   "FILE:##filename##;;##size##;;"
	# with a correct parameters hash given, message
	#   "FILE::foobar.tar.gz;;42;;" 
	# can be generated and then parsed returning a hash:
	#	  { :filename => 'foobar.tar.gz', :size => '42 }
	def initialize(format_string)
		@format_string = format_string
		@vars = format_string.scan(/##(.*?)##/).flatten
		regex_string = "^" + format_string.gsub(/##.*?##/, "(.*?)") + "$"
		@regexp = Regexp.new regex_string
	end
	def to_r
		return @regexp
	end
	def to_s
		@format_string
	end

	# parse message received on a socket an return hash of extracted
	# parameters
	def parse(rawmessage)
		match = @regexp.match(rawmessage).to_a[1..-1]
		@vars.inject({}) do |h,v|
			h[v]= match.shift; h
		end
	end

	# given a hash of paramaters, generate the raw message ready
	# to be sent on a socket
	def format(params)
		params.inject(@format_string) do |msg,hash|
			msg.gsub(Regexp.new("###{hash[0]}##"), hash[1].to_s)
		end + "\n"
	end
end

module MefabCommon
	include Logging
	@@handlers = {}
	@@messages = {}

	# a given format_string match a code block
	def add_handler(format_string, &block)
		@@handlers[bla = Message.new(format_string)] = block
	end

	# a given message type match a format_string 
	def add_message(message, format_string)
		@@messages[message] = Message.new(format_string)
	end

	# a given message type match a Message object and parameters
	# are interpolated to send the rawmessage on a socket
	def send(message, params, socket=@socket)
		unless @@messages.has_key?(message)
			log :fatal, "#{self.class}: message '#{message.inspect}' not declared"
			exit 1
		end
		try = 0
		begin
			try += 1
			socket.write(@@messages[message].format(params))
		rescue Exception => e
			log :warn, "#{self.class}: message couldn't be sent to #{socket.info}"
			log :warn, "#{self.class}: try #{try}, retrying in 5 seconds"
			sleep 5
			retry if try <= 3
		end
	end

	# a given message is received on a socket and matching code block
	# is called with parameters extracted from this rawmessage
	def handle(rawmessage,socket=@socket)
		#log :debug, "#{self.class}: handling #{rawmessage.inspect}"
		@@handlers.each do |msg,b|
			# log :debug, "#{self.class}: matching #{rawmessage.inspect} =~ #{msg.to_r.inspect} => #{(rawmessage =~ msg.to_r).inspect}"
			if rawmessage =~ msg.to_r
				b.call(msg.parse(rawmessage),socket) 
				return
			end
		end
		log :warn, "#{self.class}: message #{rawmessage.inspect} unknwon"
	end
end
